from kk_ontology_module.load_data import load_data
from kk_ontology_module.load_onto import load_onto
from kk_ontology_module.map_data import map_data
from kk_ontology_module.datamanip import to_pandas, has_icd10_code, has_morphology_code
from kk_ontology_module.reason import CancerOntology, test_data